package NpsPerlSDK::SoapClient;

use warnings; 
use strict;

use NpsPerlSDK::Configuration; 
use NpsPerlSDK::Utils; 
use NpsPerlSDK::Constants; 
use NpsPerlSDK::Errors;

use Data::Structure::Util qw( unbless ); 

my $connection; 
my $response; 
my $error;

sub _setup {
	my $self = shift;
	if (defined $NpsPerlSDK::Configuration::debug) { 
        NpsPerlSDK::LogException->error if ($NpsPerlSDK::Configuration::log_level eq $NpsPerlSDK::Constants::DEBUG and 
                                $NpsPerlSDK::Configuration::environment eq $NpsPerlSDK::Constants::PRODUCTION_ENV); 

    if ($NpsPerlSDK::Configuration::log_level > $NpsPerlSDK::Constants::DEBUG){
        eval {  require SOAP::Lite;
                SOAP::Lite->import(+trace => [ transport => sub {
                my ($http_object) = @_;
                NpsPerlSDK::Utils::masking_func($http_object, \&NpsPerlSDK::Utils::mask_data);
                }]);
             }
        } else {
        eval {  require SOAP::Lite;
                SOAP::Lite->import(+trace => [ transport => sub {
                my ($http_object) = @_;
                NpsPerlSDK::Utils::masking_func($http_object, sub{return $_[0]});
                }]);
             }
        }
	} else {
        use SOAP::Lite;
    }
    
    $connection = SOAP::Lite
        -> on_action(sub {sprintf '%s%s', @_})
        -> on_fault( sub {
            my ( $soap, $res ) = @_;
            if (index($res, "timeout") != -1) { $error = 1; }
            elsif (index($res, "connect") != -1) { $error = 2; }
        });

    if ($NpsPerlSDK::Configuration::cert_verify_peer == 1) {
        use IO::Socket::SSL;
        $ENV{PERL_LWP_SSL_VERIFY_HOSTNAME}=0;
        if (defined $NpsPerlSDK::Configuration::proxy_url) {
            $connection -> proxy(NpsPerlSDK::Configuration::get_url(),
                                 proxy => ['http' => $NpsPerlSDK::Configuration::proxy_url],
                                 timeout => $NpsPerlSDK::Configuration::timeout,
                                 ssl_opts => [ SSL_verify_mode => 0 ]);
        } else {
           $connection -> proxy(NpsPerlSDK::Configuration::get_url(),
                          timeout => $NpsPerlSDK::Configuration::timeout,
                          ssl_opts => [ SSL_verify_mode => 0 ]);
        }
    } else {
        if (defined $NpsPerlSDK::Configuration::proxy_url) {
            $connection -> proxy(NpsPerlSDK::Configuration::get_url(),
                                 proxy => ['http' => $NpsPerlSDK::Configuration::proxy_url],
                                 timeout => $NpsPerlSDK::Configuration::timeout);
        } else {
           $connection -> proxy(NpsPerlSDK::Configuration::get_url(),
                          timeout => $NpsPerlSDK::Configuration::timeout);
        }
    }
    
    if (defined $NpsPerlSDK::Configuration::certificate and defined $NpsPerlSDK::Configuration::c_key) {
        $connection->transport->ssl_opts(
            SSL_cert_file => $NpsPerlSDK::Configuration::certificate,
            SSL_key_file  => $NpsPerlSDK::Configuration::c_key
        );
    }
}

sub soap_call {
    my ($service, $ref_params) = @_;
    _setup();
    
    my $params = NpsPerlSDK::Utils::add_extra_info($service, $ref_params);
    $params = NpsPerlSDK::Utils::check_sanitize(params=>$params, is_root=>1)
        if (defined $NpsPerlSDK::Configuration::sanitize);
    $params = NpsPerlSDK::Utils::add_secure_hash($NpsPerlSDK::Configuration::secret_key, $params)
        if (!(exists $params->{"psp_ClientSession"}));
    eval {$response = $connection->$service(transform_params(("Requerimiento" => $params)));};
    
    if (defined $error) {
        if ($error == 1) {
            return NpsPerlSDK::TimeoutException->new();
        } elsif ($error == 2 ) {
            return NpsPerlSDK::ConnectionException->new();
        } else {
            return NpsPerlSDK::UnknownError->new();
        }
    } else {
        my $result = NpsPerlSDK::Utils::encode_params(unbless($response->result));
        return $result;
    }
}

sub transform_params {
    my %params = @_;
    my @params;
    foreach my $key (keys %params) {
        my $item = SOAP::Data->name($key => $params{$key});
        push(@params, $item);
    }
    return @params;
}

1;

